{% load static %}
<script defer>var reactor_components = {{ components|safe }}; </script>
<script defer src="{% static "morphdom.min.js" %}"></script>
<script defer src="{% static "reactor.js" %}"></script>
